/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ProcessStoppedException;
import com.rapidminer.operator.meta.GridSearchParameterOptimizationOperator;

/**
 * Extension of GridSearchParameterOptimizationOperator, which allows to
 * receive actual percentage of process completion via some updating mechanism.
 *
 */

public class GridSearchCount extends GridSearchParameterOptimizationOperator {

    public interface Update {
        public void update(double value);
    }
    
    protected int n;
    Update update;
    
    public GridSearchCount(OperatorDescription description) {
        super(description);
    }
    
    public void setUpdate(Update update) {
        this.update = update;
    }
    
    public void doWork() throws OperatorException {
        n = 0;
        super.doWork();
    }
    
    public void inApplyLoop() throws ProcessStoppedException {
        n++;
        if(update!=null) {
            update.update(n / (double)numberOfCombinations);
        }
        super.inApplyLoop();
    }

}
