/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.casebase;

import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.Action;
import javax.swing.Icon;

import com.rapidminer.Process;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.gui.renderer.RendererService;
import com.rapidminer.operator.AbstractIOObject;
import com.rapidminer.operator.Annotations;
import com.rapidminer.operator.ResultObject;
import com.rapidminer.operator.SimpleResultObject;
import com.rapidminer.tools.XMLException;

import de.dfki.madm.paren.gui.casebase.CaseBaseExampleSetRenderer;
import de.dfki.madm.paren.gui.casebase.CaseBaseParameterRenderer;

/**
 * This contains all relevant information for performing further meta-learning.
 * 
 *  - Accuracies for several classifiers for several datasets
 *  - Required pre-processing steps for each classifier
 *  - Required main processes which contain the classifier itself
 *  - Parameter specifications for doing parameter grid search.
 * 
 * @author Alexander Arimond
 *
 */

public class CaseBase extends AbstractIOObject implements ResultObject {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -624817534348226299L;

	public static final String ATTRIBUTE_DATASET = "Path to the dataset";

	
	private ExampleSet casebase;

	private HashMap <String, String> preprocesses = new HashMap <String, String>();
	private HashMap <String, String> processes = new HashMap <String, String>();
	//FIXME SimpleResultObject ?? or some parameter class?
	private HashMap <String, SimpleResultObject> params = new HashMap <String, SimpleResultObject> ();

	private Annotations annotations = new Annotations();
	
	// FIXME exchange this with ioobjects.xml entry
	private static boolean called = false;
	private static synchronized void registerRenderer(){
		//boolean called;
		if (called) return;
		called = true;
		final String className = CaseBase.class.getCanonicalName();
		final String superTypeName = CaseBase.class.getSuperclass().getCanonicalName();
		final boolean reportable = true;
		final List<String> rendererNames = new LinkedList<String>();
		rendererNames.add(CaseBaseExampleSetRenderer.class.getCanonicalName());
		rendererNames.add(CaseBaseParameterRenderer.class.getCanonicalName());
		//rendererNames.add(CaseBasePreprocessesRenderer.class.getCanonicalName());
		ClassLoader classLoader = CaseBase.class.getClassLoader();
		RendererService.registerRenderers(superTypeName, className, reportable, rendererNames, classLoader);
	}

	public CaseBase(ExampleSet exampleSet){
		// FIXME maybe copy??
		this.casebase = exampleSet;
		registerRenderer();
		//System.out.println(RendererService.getRenderers(this).get(0).getClass().getCanonicalName());
	}

	public ExampleSet getCasebaseExampleSet(){
		return this.casebase;
	}
	
	public void addClassifier(String name, Process preprocess, Process process, SimpleResultObject optimizationParams){
		this.processes.put(name, process.getRootOperator().getXML(false));
		this.preprocesses.put(name, preprocess.getRootOperator().getXML(false));
		this.params.put(name, optimizationParams);
	}
	
	public Process getPreprocess(String classifier){
		try {
			return new Process(this.preprocesses.get(classifier));
		} catch (IOException e) {
			// should not happen because its not read from file
			e.printStackTrace();
		} catch (XMLException e) {
			// should not happen because process is read by process.getRootOperator().getXML() before
			e.printStackTrace();
		}
		return null;
	}
	
	public Process getProcess(String classifier){
		try {
			return new Process(this.processes.get(classifier));
		} catch (IOException e) {
			// should not happen because its not read from file
			e.printStackTrace();
		} catch (XMLException e) {
			// should not happen because process is read by process.getRootOperator().getXML() before
			e.printStackTrace();
		}
		return null;
	}
	
	public SimpleResultObject getOptimizationParams(String classifier){
		return this.params.get(classifier);
	}
	
	public List<String> getClassifierNames(){
		LinkedList<String> names = new LinkedList<String>();
		Set<String> keys = this.params.keySet();
		for (String name : keys){
			names.add(name);
		}
		return names;
	}
	
	
	@Override
	public Annotations getAnnotations() {
		return this.annotations ;
	}

	@Override
	public String getName() {
		return "CaseBase";
	}

	@Override
	public String toResultString() {
		return "resultString";
	}

	@Override
	public Icon getResultIcon() {
		return null;
	}

	@Override
	public List<Action> getActions() {
		return new LinkedList<Action>();
	}

}
