/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.gui.casebase;

import java.awt.Component;
import java.util.Collection;
import java.util.logging.Level;

import javax.swing.JLabel;

import com.rapidminer.example.ExampleSet;
import com.rapidminer.gui.renderer.AbstractRenderer;
import com.rapidminer.gui.renderer.DefaultReadable;
import com.rapidminer.gui.renderer.Renderer;
import com.rapidminer.gui.renderer.RendererService;
import com.rapidminer.gui.tools.RadioCardPanel;
import com.rapidminer.operator.IOContainer;
import com.rapidminer.report.Reportable;
import com.rapidminer.tools.LogService;

import de.dfki.madm.paren.casebase.CaseBase;

public class CaseBaseExampleSetRenderer extends AbstractRenderer {

	@Override
	public String getName() {
		return "ExampleSet View";
	}

	@Override
	public Component getVisualizationComponent(Object casebase,
			IOContainer ioContainer) {
		final ExampleSet caseBaseExampleSet = ((CaseBase) casebase).getCasebaseExampleSet();
		final String resultName = RendererService.getName(ExampleSet.class);
		Collection<Renderer> renderers = RendererService.getRenderers(resultName);

		// constructing panel of renderers
		Component visualisationComponent = new RadioCardPanel("exampleset", caseBaseExampleSet);
		for (Renderer renderer : renderers) {
			try {
				Component rendererComponent = renderer.getVisualizationComponent(caseBaseExampleSet, ioContainer);
				if (rendererComponent != null)
					((RadioCardPanel)visualisationComponent).addCard(renderer.getName(), rendererComponent);				
			} catch (Exception e) {
				LogService.getRoot().log(Level.WARNING, "Error creating renderer: "+e, e);
				((RadioCardPanel)visualisationComponent).addCard(renderer.getName(), new JLabel("Error creating renderer "+renderer.getName() + " (see log)."));
			}
		}
		return visualisationComponent;
	}

	@Override
	public Reportable createReportable(Object renderable,
			IOContainer ioContainer, int desiredWidth, int desiredHeight) {
		return new DefaultReadable("ExampleSet");
	}





}
