/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.gui.casebase;

import java.awt.Component;
import java.util.List;
import java.util.logging.Level;

import javax.swing.JLabel;

import com.rapidminer.gui.renderer.AbstractRenderer;
import com.rapidminer.gui.renderer.DefaultReadable;
import com.rapidminer.gui.renderer.DefaultTextRenderer;
import com.rapidminer.gui.renderer.Renderer;
import com.rapidminer.gui.renderer.RendererService;
import com.rapidminer.gui.tools.RadioCardPanel;
import com.rapidminer.operator.IOContainer;
import com.rapidminer.operator.SimpleResultObject;
import com.rapidminer.report.Reportable;
import com.rapidminer.tools.LogService;

import de.dfki.madm.paren.casebase.CaseBase;

public class CaseBaseParameterRenderer extends AbstractRenderer {

	@Override
	public String getName() {
		// TODO Auto-generated method stub
		return "Parameters";
	}

	@Override
	public Component getVisualizationComponent(Object renderable,
			IOContainer ioContainer) {
		CaseBase casebase = (CaseBase) renderable;
		List <String> classifierNames = casebase.getClassifierNames();
		
		// FIXME do not use "casebase" here, but some kind of collection of the optimized parameters
		Component visualisationComponent = new RadioCardPanel("Optimized Parameters", casebase);
		for (String classifier : classifierNames){
			try {
				SimpleResultObject optimizationParams = casebase.getOptimizationParams(classifier);
				List <Renderer> renderers = RendererService.getRenderers(RendererService.getName(SimpleResultObject.class));
				
				// fallback to default toString method!
				if (renderers.isEmpty()) {
					renderers.add(new DefaultTextRenderer());
				}
				
				Renderer renderer = renderers.get(0);
				Component rendererComponent = renderer.getVisualizationComponent(optimizationParams, ioContainer);
				if (rendererComponent != null)
					//((RadioCardPanel)visualisationComponent).addCard(renderer.getName(), rendererComponent);
					((RadioCardPanel)visualisationComponent).addCard(classifier, rendererComponent);
			} catch (Exception e) {
				LogService.getRoot().log(Level.WARNING, "Error creating renderer: "+e, e);
				//((RadioCardPanel)visualisationComponent).addCard(renderer.getName(), new JLabel("Error creating renderer "+renderer.getName() + " (see log)."));
				((RadioCardPanel)visualisationComponent).addCard(classifier, new JLabel("Error creating renderer "+classifier + " (see log)."));
			}
		}
		return visualisationComponent;
	}

	@Override
	public Reportable createReportable(Object renderable,
			IOContainer ioContainer, int desiredWidth, int desiredHeight) {
		// TODO Auto-generated method stub
		return new DefaultReadable("Parameters");
	}

}
