/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.gui.wizard;

import java.util.HashMap;

import javax.swing.table.AbstractTableModel;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.tools.Tools;

import de.dfki.madm.paren.Regressioner;

/**
 * 
 * @author Matthias Reif
 *
 */

public class ClassifierTableModel extends AbstractTableModel {

    /**
	 * 
	 */
	private static final long serialVersionUID = -4350774461195655604L;
	private String[] columnNames = { "Classifier", "Predicted Accuracy", "Root Mean Squared Error", "Evaluate" };
    Attribute classifierAttribute, predictionAttribute, rmseAttribute;
    private HashMap<Double, Boolean> evaluate;
    private ExampleSet predictions;

    public ClassifierTableModel(ExampleSet predictions) {
        this.predictions = predictions;
        classifierAttribute = predictions.getAttributes().get(Regressioner.ATTRIBUTE_CLASSIFIER);
        predictionAttribute = predictions.getAttributes().get(Regressioner.ATTRIBUTE_PREDICTION);
        rmseAttribute = predictions.getAttributes().get(Regressioner.ATTRIBUTE_RMSE);
        evaluate = new HashMap<Double, Boolean>();
        double maxId = 0;
        double maxValue = 0.0;
        for(Example example : predictions) {
            double value = example.getNumericalValue(predictionAttribute);
            double id = example.getId();
            evaluate.put(id, Boolean.FALSE);
            if(value > maxValue) {
                maxValue = value;
                maxId = id;
            }
        }
        evaluate.put(maxId, Boolean.TRUE);
    }

    public ExampleSet getPredictions() {
        return predictions;
    }

    public HashMap<Double, Boolean> getEvaluations() {
        return evaluate;
    }

    @Override
    public int getColumnCount() {
        return columnNames.length;
    }

    @Override
    public int getRowCount() {
        return predictions.size();
    }

    @Override
    public String getColumnName(int col) {
        return columnNames[col];
    }

    @Override
    public Object getValueAt(int row, int col) {
        Example example = predictions.getExample(row);
        switch(col) {
        case 0:
            return example.getNominalValue(classifierAttribute);
        case 1:
            double prediction = example.getNumericalValue(predictionAttribute);
            if (prediction > 1.0) prediction = 1.0; // This may happen because of regression modelling, but looks strange to the user
            return Tools.formatNumber(prediction);
        case 2:
            return Tools.formatNumber(example.getNumericalValue(rmseAttribute));
        case 3:
            return evaluate.get(example.getId());
        }
        return null;
    }

    public Class <? extends Object> getColumnClass(int col) {
        if(col == 3) {
            return Boolean.class;
        } else if(col == 4) {
            return Object.class;
        } else {
            return String.class;
        }
    }

    public boolean isCellEditable(int row, int col) {
        return col == 3;
    }

    public void setValueAt(Object value, int row, int col) {
        if(col == 3) {
            Example example = predictions.getExample(row);
            evaluate.put(example.getId(), (Boolean) value);
        }
        fireTableCellUpdated(row, col);
    }

}
