/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.landmarking;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;

import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.set.AttributeSelectionExampleSet;
import com.rapidminer.operator.OperatorCreationException;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.learner.tree.DecisionStumpLearner;
import com.rapidminer.operator.performance.PerformanceVector;
import com.rapidminer.tools.OperatorService;
import com.rapidminer.tools.math.RunVector;

/**
 * This landmark learner is an Average landmark learner, where a
 * {@link DecisionStumpLearner} for each attribute is created and then the
 * performance average is calculated.
 * 
 * @author Sarah Daniel
 * 
 */
public class AverageNodeLm extends AbstractLandmarker {
    
    
    public AverageNodeLm(ExampleSet exampleSet, Map<String, String> parameters) throws OperatorException, OperatorCreationException {
        this.name = LandmarkingParameters.PARAMETER_AVERAGE_NODE_LM;

        exampleSet = normalizeExampleSet(exampleSet, parameters);
        calculateAverageStumpAccuray(exampleSet, parameters);
    }

    /**
     * A {@link DecisionStumpLearner} for each attribute of the
     * {@link ExampleSet} is created and its performance is evaluated. Then the
     * average of all calculated performances is retrieved using the
     * {@link RunVector}.
     * 
     * @param exampleSet
     *            The {@link ExampleSet} to be evaluated.
     * @param parameters
     *            The {@link Map} containing all parameters of the landmarking.
     */
    private void calculateAverageStumpAccuray(ExampleSet exampleSet, Map<String, String> parameters) {
        try {

            int numberOfAttributes = exampleSet.getAttributes().size();
            boolean[] selectedAttributes = null;
            AttributeSelectionExampleSet selectedExampleSet = null;
            DecisionStumpLearner learner = OperatorService.createOperator(DecisionStumpLearner.class);

            String learningMethodName = null;
            if (Boolean.valueOf(parameters.get(LandmarkingParameters.PARAMETER_CROSS_VALIDATION)))
                learningMethodName = "xValidation";
            else
                learningMethodName = "learn";

            Method method = AbstractLandmarker.class.getDeclaredMethod(learningMethodName, new Class[] { com.rapidminer.example.ExampleSet.class, com.rapidminer.operator.Operator.class, Map.class });
            RunVector runVector = new RunVector();

            for (int count = 0; count < numberOfAttributes; count++) {
                learner = OperatorService.createOperator(DecisionStumpLearner.class);

                selectedAttributes = new boolean[numberOfAttributes];
                selectedAttributes[count] = true;
                selectedExampleSet = new AttributeSelectionExampleSet(exampleSet, selectedAttributes);

                method.invoke(this, new Object[] { selectedExampleSet, learner, parameters });
                PerformanceVector tempPerformanceVector = this.performance;
                runVector.addVector(tempPerformanceVector);
            }

            this.performance = (PerformanceVector) runVector.average();
        } catch (SecurityException e) {

            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (OperatorCreationException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
    }
}
