/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.landmarking;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.rapidminer.operator.ResultObjectAdapter;

/**
 * This class is a {@link ResultObjectAdapter}, where the landmark results are
 * contained.
 * 
 * @author Sarah Daniel
 * 
 */
public class LandmarkingResults {
    /**
     * Should contain a {@link String} as key for the landmark learner name and
     * an {@link Object} containing the performance result of the landmarker.
     * 
     */
    private Map<String, Object> landmarkersResult = new HashMap<String, Object>();

    /**
     * @return array containing the names of the landmarkers.
     */
    public String[] landmarkerNames() {
        Set<String> keySet = this.landmarkersResult.keySet();
        return Arrays.copyOf(keySet.toArray(), keySet.size(), String[].class);
    }

    /**
     * @return an <code>Object[][]</code> containing the landmark results.
     */
    public Object[][] getPerformanceData() {
        Object[][] data = new Object[LandmarkingParameters.NUMBER_OF_PERFORMANCE_CRITERIA][];
        data[0] = landmarkersResult.values().toArray();
        return data;
    }

    /**Adds a new entry to the {@link Map} {@ landmarkersResult.
	 * 
	 * @param key The name of the landmark learner. 
	 * @param value The {@Object} of criteria value/s of the landmark learner.
	 */
    public void addLandMarker(String key, Object value) {
        landmarkersResult.put(key, value);
    }

    /**
     * @return The {@link Map} containing the landmark results.
     */
    public Map<String, Object> getResult() {
        return landmarkersResult;
    }

}
