/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.landmarking;

import java.util.Map;

import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.OperatorCreationException;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.learner.bayes.LinearDiscriminantAnalysis;
import com.rapidminer.operator.preprocessing.filter.NominalToBinominal;
import com.rapidminer.operator.preprocessing.filter.NominalToNumeric;
import com.rapidminer.tools.LogService;
import com.rapidminer.tools.OperatorService;

/**
 * This landmark learner is a {@link LinearDiscriminantAnalysis} learner. This
 * landmark can not be evaluated using cross-validation.
 * 
 * @author Sarah Daniel
 * 
 */

public class LinearDiscriminantLm extends AbstractLandmarker {

    
    /**
     * To avoid erros of the {@link LinearDiscriminantAnalysis} operator the
     * {@link ExampleSet} is preprocessed using the following preprocessing
     * operators:
     * <p>
     * {@link TrimMapping}, {@link NominalToBinominal}, {@link NominalToNumeric}
     * </p>
     */
    public LinearDiscriminantLm(ExampleSet exampleSet, Map<String, String> parameters) {
        try {
            this.name = LandmarkingParameters.PARAMETER_LINEAR_DISCRIMINANT_LM;

            TrimMapping trimMapping = OperatorService.createOperator(TrimMapping.class);
            ExampleSet trimmedExampleSet = trimMapping.apply(exampleSet);

            NominalToBinominal nominalToBinominal = OperatorService.createOperator(NominalToBinominal.class);
            nominalToBinominal.createPreprocessingModel(trimmedExampleSet);
            ExampleSet binominalExampleSet = nominalToBinominal.doWork(trimmedExampleSet);

            NominalToNumeric nominalToNumeric = OperatorService.createOperator(NominalToNumeric.class);
            nominalToNumeric.createPreprocessingModel(binominalExampleSet);
            ExampleSet numericExampleSet = nominalToNumeric.doWork(binominalExampleSet);

            if (Boolean.valueOf(parameters.get(LandmarkingParameters.PARAMETER_CROSS_VALIDATION)))
                LogService.getRoot().info("The LDA landmark will be evaluated without using cross-validation.");

            LinearDiscriminantAnalysis learner = OperatorService.createOperator(LinearDiscriminantAnalysis.class);
            numericExampleSet = normalizeExampleSet(numericExampleSet, parameters);

            learn(numericExampleSet, learner, parameters);

        } catch (OperatorCreationException e) {
            e.printStackTrace();
        } catch (OperatorException e) {
            e.printStackTrace();
        }
    }

}
