/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.landmarking;

import java.util.Map;

import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.set.AttributeSelectionExampleSet;
import com.rapidminer.operator.OperatorCreationException;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.learner.tree.DecisionStumpLearner;
import com.rapidminer.tools.OperatorService;

/**
 * This landmark learner is a {@link DecisionStumpLearner}, where a single
 * decision node is chosen randomly.
 * 
 * @author Sarah Daniel
 * 
 */
public class RandomlyChosenNodeLm extends AbstractLandmarker {
    public RandomlyChosenNodeLm(ExampleSet exampleSet, Map<String, String> parameters) throws OperatorException, OperatorCreationException {


            this.name = LandmarkingParameters.PARAMETER_RANDOMLY_CHOSEN_NODE_LM;

            DecisionStumpLearner learner = OperatorService.createOperator(DecisionStumpLearner.class);

            AttributeSelectionExampleSet attributeSelectionExampleSet = getAttributeSelectionExampleset(exampleSet);
            exampleSet = normalizeExampleSet(attributeSelectionExampleSet, parameters);

            learnExampleSet(exampleSet, learner, parameters);

    }

    /**
     * @param exampleSet
     *            The {@link ExampleSet} to be modified.
     * @return a new {@link AttributeSelectionExampleSet}, where the selected
     *         attribute is chosen randomly using @ #selectNumberRandomly(int)}
     *         .
     */
    private AttributeSelectionExampleSet getAttributeSelectionExampleset(ExampleSet exampleSet) {
        int numberOfAttributes = exampleSet.getAttributes().size();
        boolean[] selectedAttributes = new boolean[numberOfAttributes];
        int randomSelection = selectNumberRandomly(numberOfAttributes - (numberOfAttributes == 0 ? 0 : 1));
        selectedAttributes[randomSelection] = true;

        return new AttributeSelectionExampleSet(exampleSet, selectedAttributes);
    }

    /**
     * @param maximum
     *            The maximum number to be returned.
     * @return Returns a number in the range from 0 to maximum.
     */
    private int selectNumberRandomly(int maximum) {
        return (int) (Math.random() * maximum);
    }
}
