/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.landmarking;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.operator.AbstractExampleSetProcessing;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;

/**
 * @brief removes mappings of label values that actually does not occur replaces
 *        the old label column with a new one
 * @author remat
 */
public class TrimMapping extends AbstractExampleSetProcessing {

    public TrimMapping(OperatorDescription description) {
        super(description);
    }

    @Override
    public ExampleSet apply(ExampleSet exampleSet) throws OperatorException {
        Attribute oldLabel = exampleSet.getAttributes().getLabel();
        // -- add new attribute and copy values --
        Attribute newLabel = AttributeFactory.createAttribute(oldLabel.getName() + " new", oldLabel.getValueType());
        exampleSet.getExampleTable().addAttribute(newLabel);
        exampleSet.getAttributes().addRegular(newLabel);
        for (Example example : exampleSet) {
            example.setValue(newLabel, example.getNominalValue(oldLabel));
        }
        // -- remove old label --
        exampleSet.getExampleTable().removeAttribute(oldLabel);
        exampleSet.getAttributes().remove(oldLabel);
        exampleSet.getAttributes().setLabel(newLabel);
        newLabel.setName(oldLabel.getName());
        return exampleSet;
    }

}
