/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.paren.util;


import java.util.List;

import com.rapidminer.operator.ExecutionUnit;
import com.rapidminer.operator.IOContainer;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorChain;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.Port;
import com.rapidminer.operator.ports.PortException;
import com.rapidminer.operator.ports.PortExtender;
import com.rapidminer.operator.ports.PortOwner;
import com.rapidminer.operator.ports.Ports;
import com.rapidminer.operator.ports.impl.OutputPortImpl;
import com.rapidminer.tools.Observer;

/**
 * This is a workaround to make access to InputPorts possible which are 
 * "hidden" by an <code>InputPortExtender</code>. By this you can access the InputPorts of
 * an InputPortExtender-controlled Operator directly and deliver data on the ports 
 * programatically instead of connecting from some other Operator in the Process view.
 * 
 * @author Alexander Arimond
 */

public class ConnectToInputPortExtender{

	private Operator operatorWithInputPortExtender;
	public ConnectToInputPortExtender(Operator operatorWithInputPortExtender){
		// FIXME this is not "type safe" because you dont know if the operator has an inputportextender
		this.operatorWithInputPortExtender = operatorWithInputPortExtender;
		connectionContext = operatorWithInputPortExtender.getInputPorts().getOwner().getConnectionContext();
	}

	public InputPort connectToNextInputPort(){
		InputPort inputPort = this.operatorWithInputPortExtender.getInputPorts().getPortByIndex(this.operatorWithInputPortExtender.getInputPorts().getNumberOfConnectedPorts());
		DummyOutputPort dummyOutput = new DummyOutputPort(dummyOwner, "dummy", false);
		dummyOutput.connectTo(inputPort);
		return inputPort;
	}
	
	class DummyOutputPort extends OutputPortImpl{

		protected DummyOutputPort(Ports<? extends Port> owner, String name,
				boolean simulatesStack) {
			super(owner, name, simulatesStack);
		}
		
		public boolean isConnected(){
			return false;
		}
		
	}

	ExecutionUnit connectionContext;
	
	// this is a mock object for Ports owner for simulating an Operator which delivers output to a InputPortExtender..
	// it just is used to make .connectTo() call possible and is strongly aligned to this method..
	
	Ports<? extends Port> dummyOwner = new Ports<DummyOutputPort>(){

		// we need the getOwner() method implemented for the dummy here.
		@Override
		public PortOwner getOwner() {
			return new PortOwner() {
				
				// we need the getConnectionContext() method here. it just returns the same connectionContext as the
				// Operator it will connect to
				@Override
				public ExecutionUnit getConnectionContext() {
					return connectionContext;
				}
				
				// The rest is just empty Mock code for PortOwner dummy
				@Override
				public OperatorChain getPortHandler() {
					return null;
				}
				@Override
				public Operator getOperator() {
					return null;
				}
				@Override
				public String getName() {
					return null;
				}
			};
		}
		
		
		// The rest is just empty Mock code for Ports dummy
		@Override
		public void addObserver(Observer<Port> observer, boolean onEDT) {			
		}
		@Override
		public void removeObserver(Observer<Port> observer) {		
		}
		@Override
		public void addObserverAsFirst(Observer<Port> observer,
				boolean onEDT) {			
		}
		@Override
		public String[] getPortNames() {
			return null;
		}
		@Override
		public int getNumberOfPorts() {
			return 0;
		}
		@Override
		public void removeAll() throws PortException {			
		}
		@Override
		public String getMetaDataDescription() {
			return null;
		}
		@Override
		public void clear(int clearFlags) {
		}
		@Override
		public IOContainer createIOContainer(boolean onlyConnected) {
			return null;
		}
		@Override
		public void disconnectAll() {	
		}
		@Override
		public void registerPortExtender(PortExtender extender) {
		}
		@Override
		public void unlockPortExtenders() {
		}
		@Override
		public void freeMemory() {
		}
		@Override
		public int getNumberOfConnectedPorts() {
			return 0;
		}
		@Override
		public DummyOutputPort getPortByName(String name) {
			return null;
		}
		@Override
		public DummyOutputPort getPortByIndex(int index) {
			return null;
		}
		@Override
		public List<DummyOutputPort> getAllPorts() {
			return null;
		}
		@Override
		public void addPort(DummyOutputPort port) throws PortException {			
		}
		@Override
		public void removePort(DummyOutputPort port) throws PortException {			
		}
		@Override
		public boolean containsPort(DummyOutputPort port) {
			return false;
		}
		@Override
		public DummyOutputPort createPort(String name) {
			return null;
		}
		@Override
		public DummyOutputPort createPassThroughPort(String name) {
			return null;
		}
		@Override
		public DummyOutputPort createPort(String name, boolean add) {
			return null;
		}
		@Override
		public void renamePort(DummyOutputPort port, String newName) {
		}
		@Override
		public void pushDown(DummyOutputPort port) {
		}
	};
}
